@@ -21,12 +21,19 @@
  * License along with this library; if not, write to the
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
+ *
+ * Note March 9, 2003: I fixed a crash with regards to precomposed
+ * characters, by wraping all of them to be considered as ALEF as
+ * far as consideration about composability is concerned. The rendering
+ * with regards to precomposed characters AND nikud comes out really
+ * bad though, and should be fixed, once I have more time.
  */
 
 #include <glib.h>
 #include "pango-engine.h"
 
-#define ucs2iso8859_8(wc)		(unsigned int)((unsigned int)(wc) - 0x0590 + 0x10)
+/* Wrap all characters above 0xF00 to ALEF. */
+#define ucs2iso8859_8(wc)		(wc>0xF000 ? 0x11 : (unsigned int)((unsigned int)(wc) - 0x0590 + 0x10))
 #define iso8859_8_2uni(c)		((gunichar)(c) - 0x10 + 0x0590)
 
 #define MAX_CLUSTER_CHRS	256
@@ -61,15 +68,19 @@
 #define UNI_SHIN		0x05E9
 #define UNI_FINAL_PE		0x05E3
 #define UNI_PE			0x05E4
+#define UNI_QOF                 0x05E7
 #define	UNI_TAV			0x05EA
 #define UNI_SHIN_DOT		0x05C1
 #define UNI_SIN_DOT		0x05C2
 #define UNI_MAPIQ		0x05BC
 #define	UNI_SHEVA		0x05B0
-#define	UNI_QAMATS		0x05B8
 #define	UNI_HOLAM		0x05B9
 #define	UNI_QUBUTS		0x05BB
-
+#define UNI_HATAF_SEGOL         0x05B1
+#define UNI_HATAF_QAMATZ        0x05B3
+#define UNI_TSERE               0x05B5
+#define UNI_QAMATS              0x05B8
+#define UNI_QUBUTS              0x05BB
 
 /*======================================================================
 //  In the tables below all Hebrew characters are categorized to
@@ -195,7 +206,9 @@ static const gint Unicode_shape_table[128] = {
               0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000, 0x0000,
 };
 
-#define is_char_class(wc, mask)	(char_class_table[ucs2iso8859_8 ((wc))] & (mask))
+/* Treat all characters above 0xF000 as characters */
+#define is_char_class(wc, mask)	(wc > 0xF000 \
+                                 || char_class_table[ucs2iso8859_8 ((wc))] & (mask))
 #define	is_composible(cur_wc, nxt_wc)	(compose_table[char_type_table[ucs2iso8859_8 (cur_wc)]]\
 						      [char_type_table[ucs2iso8859_8 (nxt_wc)]])
 
@@ -249,9 +262,20 @@ hebrew_shaper_get_cluster_kerning(gunichar            *cluster,
 
   x_offset[0] = 0;
   y_offset[0] = 0;
-  
+
   if (cluster_length == 1)
-    return;
+    {
+      /* Make lone 'vav dot' have zero width */
+      if (base_char == UNI_SHIN_DOT
+	  || base_char == UNI_SIN_DOT
+	  || base_char == UNI_HOLAM
+	  ) {
+	x_offset[0] = -ink_rect[0].x - ink_rect[0].width;
+	width[0] = 0;
+      }
+	
+      return;
+    }
 
   base_ink_x_offset = ink_rect[0].x;
   base_ink_y_offset = ink_rect[0].y;
@@ -319,14 +343,18 @@ hebrew_shaper_get_cluster_kerning(gunichar            *cluster,
 	  x_offset[i] = base_ink_x_offset -ink_rect[i].x - ink_rect[i].width * 3/ 2;
 	}
       
-      /* VOWELS under resh or vav are right aligned */
+      /* VOWELS under resh or vav are right aligned, if they are
+	 narrower than the characters. Otherwise they are centered.
+       */
       else if ((base_char == UNI_VAV
 		|| base_char == UNI_RESH
 		|| base_char == UNI_YOD
 		|| base_char == UNI_DALED
 		)
 	       && ((gl >= UNI_SHEVA && gl <= UNI_QAMATS) ||
-		   gl == UNI_QUBUTS)) 
+		   gl == UNI_QUBUTS)
+	       && ink_rect[i].width < base_ink_width
+	       ) 
 	{
 	  x_offset[i] = base_ink_x_offset + base_ink_width
 	    - ink_rect[i].x - ink_rect[i].width;
@@ -421,6 +449,18 @@ hebrew_shaper_get_cluster_kerning(gunichar            *cluster,
 	    + base_ink_width * 3/8 - ink_rect[i].width/2;
 	}
 
+      /* Right align wide nikud under QOF */
+      else if (base_char == UNI_QOF &&
+	       ( (gl >= UNI_HATAF_SEGOL
+		  && gl <= UNI_HATAF_QAMATZ)
+		 || (gl >= UNI_TSERE
+		     && gl<= UNI_QAMATS)
+		 || (gl == UNI_QUBUTS)))
+	{
+	  x_offset[i] = base_ink_x_offset + base_ink_width
+	    - ink_rect[i].x - ink_rect[i].width;
+	}
+
       /* Center by default */
       else
 	{  
