@@ -12,6 +12,7 @@
 #include <freetype/freetype.h>
 #include <pango/pango-glyph.h>
 #include <pango/pango-types.h>
+#include "mprefixups.h"
 
 G_BEGIN_DECLS
 
@@ -58,7 +59,7 @@ typedef enum
 {
   CF_CLASS_MASK   = 0x0000FFFF,
 
-  CF_CONSONANT    = 0x80000000,
+  CF_CONSONANT    = (int)0x80000000,
 
   CF_REPH         = 0x40000000,
   CF_VATTU        = 0x20000000,
@@ -85,7 +86,7 @@ typedef glong IndicOTCharClass;
  */
 typedef enum
 {
-  SF_MATRAS_AFTER_BASE    = 0x80000000,
+  SF_MATRAS_AFTER_BASE    = (int)0x80000000,
   SF_REPH_AFTER_BELOW     = 0x40000000,
   SF_EYELASH_RA           = 0x20000000,
   SF_MPRE_FIXUP           = 0x10000000,
@@ -133,7 +134,7 @@ enum indic_glyph_property_
   rphf_p = (junk | dist),
   blwf_p = (junk | dist | rphf),
   half_p = (junk | dist | rphf | blwf),
-  nukt_p = (junk | dist | rphf | blwf | half)
+  nukt_p = (junk | dist | rphf | blwf | half | pstf)
 };
 
 /*
@@ -241,7 +242,7 @@ gboolean indic_ot_has_below_base_form(const IndicOTClassTable *class_table, guni
 
 glong indic_ot_find_syllable(const IndicOTClassTable *class_table, const gunichar *chars, glong prev, glong char_count);
 
-glong indic_ot_reorder(const gunichar *chars, const glong *utf8_offsets, glong char_count, const IndicOTClassTable *class_table, gunichar *out_chars, glong *char_indices, gulong *char_tags);
+glong indic_ot_reorder(const gunichar *chars, const glong *utf8_offsets, glong char_count, const IndicOTClassTable *class_table, gunichar *out_chars, glong *char_indices, gulong *char_tags, MPreFixups **outMPreFixups);
 
 #endif /* PANGO_ENABLE_ENGINE */
 
