@@ -1,7 +1,9 @@
 #ifndef __THAI_SHAPER_H__
 #define __THAI_SHAPER_H__
 
+#ifdef HAVE_X
 #include "pangox.h"
+#endif
 
 #define ucs2tis(wc)     (unsigned int)((unsigned int)(wc) - 0x0E00 + 0xA0)
 #define tis2uni(c)      ((gunichar)(c) - 0xA0 + 0x0E00)
@@ -28,26 +30,25 @@ struct _ThaiFontInfo
 {
   PangoFont       *font;
   ThaiFontSet      font_set;
+#ifdef HAVE_X
   PangoXSubfont    subfont; /* For X backend */
+#endif
+
+  PangoGlyph
+  (*make_glyph) (ThaiFontInfo *font_info, unsigned int c);
+
+  PangoGlyph
+  (*make_unknown_glyph) (ThaiFontInfo *font_info, unsigned int c);
+	
+  gboolean
+  (*has_glyph) (ThaiFontInfo *font_info, PangoGlyph glyph);
 };
 
-/*
- * Abstract methods (implemented by each shaper module)
- */
-ThaiFontInfo *
-thai_get_font_info (PangoFont *font);
-
-PangoGlyph
-thai_make_glyph (ThaiFontInfo *font_info, unsigned int c);
-
-PangoGlyph
-thai_make_unknown_glyph (ThaiFontInfo *font_info, unsigned int c);
-
 /*
  * Public functions
  */
 void
-thai_engine_shape (PangoFont        *font,
+thai_engine_shape (ThaiFontInfo     *font_info,
                    const char       *text,
                    gint              length,
                    PangoAnalysis    *analysis,
