@@ -24,7 +24,7 @@
 
 #include <pango/pango-attributes.h>
 #include <pango/pango-context.h>
-#include <pango/pango-glyph.h>
+#include <pango/pango-glyph-item.h>
 #include <pango/pango-tabs.h>
 
 G_BEGIN_DECLS
@@ -32,7 +32,9 @@ G_BEGIN_DECLS
 typedef struct _PangoLayout      PangoLayout;
 typedef struct _PangoLayoutClass PangoLayoutClass;
 typedef struct _PangoLayoutLine  PangoLayoutLine;
-typedef struct _PangoLayoutRun   PangoLayoutRun;
+
+/* For backwards compatiblity */
+typedef PangoGlyphItem PangoLayoutRun;
 
 typedef enum {
   PANGO_ALIGN_LEFT,
@@ -54,12 +56,6 @@ struct _PangoLayoutLine
   GSList      *runs;
 };
 
-struct _PangoLayoutRun
-{
-  PangoItem        *item;
-  PangoGlyphString *glyphs;
-};
-
 #define PANGO_TYPE_LAYOUT              (pango_layout_get_type ())
 #define PANGO_LAYOUT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_LAYOUT, PangoLayout))
 #define PANGO_LAYOUT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_LAYOUT, PangoLayoutClass))
@@ -134,10 +130,10 @@ void     pango_layout_get_log_attrs (PangoLayout    *layout,
 				     gint           *n_attrs);
 
 void     pango_layout_index_to_pos         (PangoLayout    *layout,
-					    int             index,
+					    int             index_,
 					    PangoRectangle *pos);
 void     pango_layout_get_cursor_pos       (PangoLayout    *layout,
-					    int             index,
+					    int             index_,
 					    PangoRectangle *strong_pos,
 					    PangoRectangle *weak_pos);
 void     pango_layout_move_cursor_visually (PangoLayout    *layout,
@@ -150,7 +146,7 @@ void     pango_layout_move_cursor_visually (PangoLayout    *layout,
 gboolean pango_layout_xy_to_index          (PangoLayout    *layout,
 					    int             x,
 					    int             y,
-					    int            *index,
+					    int            *index_,
 					    int            *trailing);
 void     pango_layout_get_extents          (PangoLayout    *layout,
 					    PangoRectangle *ink_rect,
@@ -174,10 +170,10 @@ void     pango_layout_line_ref          (PangoLayoutLine  *line);
 void     pango_layout_line_unref        (PangoLayoutLine  *line);
 gboolean pango_layout_line_x_to_index   (PangoLayoutLine  *line,
 					 int               x_pos,
-					 int              *index,
+					 int              *index_,
 					 int              *trailing);
 void     pango_layout_line_index_to_x   (PangoLayoutLine  *line,
-					 int               index,
+					 int               index_,
 					 gboolean          trailing,
 					 int              *x_pos);
 void     pango_layout_line_get_x_ranges (PangoLayoutLine  *line,
@@ -222,8 +218,8 @@ void pango_layout_iter_get_line_extents    (PangoLayoutIter *iter,
  * assign between-line spacing to the nearest line)
  */
 void pango_layout_iter_get_line_yrange     (PangoLayoutIter *iter,
-                                            int             *y0,
-                                            int             *y1);
+                                            int             *y0_,
+                                            int             *y1_);
 void pango_layout_iter_get_layout_extents  (PangoLayoutIter *iter,
                                             PangoRectangle  *ink_rect,
                                             PangoRectangle  *logical_rect);
