@@ -62,7 +62,8 @@ struct _PangoFT2FontMap
   GHashTable *coverage_hash; /* Maps font file name -> PangoCoverage */
 
   GHashTable *fonts; /* Maps XftPattern -> PangoFT2Font */
-  GQueue *freed_fonts; /* Fonts in fonts that has been freed */
+	
+  GQueue *fontset_cache;	/* Recently used fontsets */
 
   /* List of all families availible */
   PangoFT2Family **families;
@@ -95,8 +96,6 @@ struct _PangoFT2FontMap
 #define pango_fc_font_map_get_type pango_ft2_font_map_get_type
 #define _pango_fc_font_map_add _pango_ft2_font_map_add
 #define _pango_fc_font_map_remove _pango_ft2_font_map_remove
-#define _pango_fc_font_map_cache_add _pango_ft2_font_map_cache_add
-#define _pango_fc_font_map_cache_remove _pango_ft2_font_map_cache_remove
 #define _pango_fc_font_map_get_coverage _pango_ft2_font_map_get_coverage
 #define _pango_fc_font_map_set_coverage _pango_ft2_font_map_set_coverage
 #define _pango_fc_font_desc_from_pattern _pango_ft2_font_desc_from_pattern
@@ -119,10 +118,12 @@ static PangoFT2FontMap *pango_ft2_global_fontmap = NULL;
  * to cache information about available fonts, and holds
  * certain global parameters such as the resolution and
  * the default substitute function (see
- * pango_font_map_set_default_substitute ()).
+ * pango_ft2_font_map_set_default_substitute()).
  * 
  * Return value: the newly created fontmap object. Unref
- * with g_object_unref when you are finished with it.
+ * with g_object_unref() when you are finished with it.
+ *
+ * Since: 1.2
  **/
 PangoFontMap *
 pango_ft2_font_map_new (void)
@@ -157,6 +158,37 @@ pango_ft2_font_map_new (void)
   return (PangoFontMap *)ft2fontmap;
 }
 
+/**
+ * pango_ft2_font_map_set_default_substitute:
+ * @fontmap: a #PangoFT2FontMap
+ * @func: function to call to to do final config tweaking
+ *        on #FcPattern objects.
+ * @data: data to pass to @func
+ * @notify: function to call when @data is no longer used.
+ * 
+ * Sets a function that will be called to do final configuration
+ * substitution on a #FcPattern before it is used to load
+ * the font. This function can be used to do things like set
+ * hinting and antialiasing options.
+ *
+ * Since: 1.2
+ **/
+void
+pango_ft2_font_map_set_default_substitute (PangoFT2FontMap        *fontmap,
+					   PangoFT2SubstituteFunc  func,
+					   gpointer                data,
+					   GDestroyNotify          notify)
+{
+  if (fontmap->substitute_destroy)
+    fontmap->substitute_destroy (fontmap->substitute_data);
+
+  fontmap->substitute_func = func;
+  fontmap->substitute_data = data;
+  fontmap->substitute_destroy = notify;
+  
+  pango_fc_font_map_cache_clear (fontmap);
+}
+
 /**
  * pango_ft2_font_map_substitute_changed:
  * @fontmap: a #PangoFT2Fontmap
@@ -166,11 +198,13 @@ pango_ft2_font_map_new (void)
  * pango_ft2_font_map_set_default_substitute() change.
  * That is, if your subsitution function will return different
  * results for the same input pattern, you must call this function.
+ *
+ * Since: 1.2
  **/
 void
 pango_ft2_font_map_substitute_changed (PangoFT2FontMap *fontmap)
 {
-  pango_fc_clear_fontset_hash_list (fontmap);
+  pango_fc_font_map_cache_clear (fontmap);
 }
 
 /**
@@ -201,6 +235,8 @@ pango_ft2_font_map_set_resolution (PangoFT2FontMap *fontmap,
  * Create a #PangoContext for the given fontmap.
  * 
  * Return value: the newly created context; free with g_object_unref().
+ *
+ * Since: 1.2
  **/
 PangoContext *
 pango_ft2_font_map_create_context (PangoFT2FontMap *fontmap)
@@ -289,6 +325,9 @@ _pango_ft2_font_map_get_library (PangoFontMap *fontmap)
 static void
 pango_fc_do_finalize (PangoFT2FontMap    *fontmap)
 {
+  if (fontmap->substitute_destroy)
+    fontmap->substitute_destroy (fontmap->substitute_data);
+
   FT_Done_FreeType (fontmap->library);
 }
 
@@ -303,7 +342,7 @@ pango_fc_default_substitute (PangoFT2FontMap *fontmap,
   if (fontmap->substitute_func)
     fontmap->substitute_func (pattern, fontmap->substitute_data);
 
-  FcDefaultSubstitute (pattern);
   if (FcPatternGet (pattern, FC_DPI, 0, &v) == FcResultNoMatch)
     FcPatternAddDouble (pattern, FC_DPI, fontmap->dpi_y);
+  FcDefaultSubstitute (pattern);
 }
