@@ -25,7 +25,7 @@
 
 #include "pango-modules.h"
 #include "pangoft2.h"
-#include "mini-xft/MiniXft.h"
+#include <fontconfig/fontconfig.h>
 
 /* Debugging... */
 /*#define DEBUGGING 1*/
@@ -55,25 +55,22 @@
 
 typedef struct _PangoFT2Font      PangoFT2Font;
 typedef struct _PangoFT2GlyphInfo PangoFT2GlyphInfo;
-typedef struct _PangoFT2Face      PangoFT2Face;
-typedef struct _PangoFT2Family    PangoFT2Family;
 
 
 struct _PangoFT2Font
 {
   PangoFont font;
 
-  MiniXftPattern *font_pattern;
+  FcPattern *font_pattern;
   FT_Face face;
+  int load_flags;
 
   int size;
 
   PangoFontMap *fontmap;
   PangoFontDescription *description;
   
-  /* If TRUE, font is in cache of recently unused fonts and not otherwise
-   * in use. */
-  gboolean in_cache;
+  GSList *metrics_by_lang;
 
   GHashTable *glyph_info;
   GDestroyNotify glyph_cache_destroy;
@@ -86,35 +83,18 @@ struct _PangoFT2GlyphInfo
   void *cached_glyph;
 };
 
-struct _PangoFT2Face
-{
-  PangoFontFace parent_instance;
-  
-  PangoFT2Family *family;
-  char *style;
-};
-
 PangoFT2Font * _pango_ft2_font_new                (PangoFontMap                *font,
-						   MiniXftPattern              *pattern);
+						   FcPattern              *pattern);
 PangoMap      *_pango_ft2_get_shaper_map          (PangoLanguage     *language);
-void           _pango_ft2_font_map_set_coverage   (PangoFontMap                *fontmap,
-						   const char                  *name,
-						   PangoCoverage               *coverage);
 PangoCoverage *_pango_ft2_font_map_get_coverage   (PangoFontMap                *fontmap,
-						   const char                  *name);
-void           _pango_ft2_face_remove             (PangoFT2Face      *face,
-						   PangoFont         *font);
+						   FcPattern                   *pattern);
 FT_Library     _pango_ft2_font_map_get_library    (PangoFontMap      *fontmap);
-void           _pango_ft2_font_map_cache_add      (PangoFontMap      *fontmap,
-						   PangoFT2Font      *ft2font);
-void           _pango_ft2_font_map_cache_remove    (PangoFontMap      *fontmap,
-						   PangoFT2Font      *ft2font);
 void           _pango_ft2_font_map_add            (PangoFontMap      *fontmap,
 						   PangoFT2Font      *ft2font);
 void           _pango_ft2_font_map_remove         (PangoFontMap      *fontmap,
 						   PangoFT2Font      *ft2font);
 const char    *_pango_ft2_ft_strerror             (FT_Error           error);
-PangoFontDescription *_pango_ft2_font_desc_from_pattern (MiniXftPattern *pattern,
+PangoFontDescription *_pango_ft2_font_desc_from_pattern (FcPattern *pattern,
 							 gboolean        include_size);
 
 void *pango_ft2_font_get_cache_glyph_data    (PangoFont      *font,
@@ -125,6 +105,4 @@ void  pango_ft2_font_set_cache_glyph_data    (PangoFont      *font,
 void  pango_ft2_font_set_glyph_cache_destroy (PangoFont      *font,
 					      GDestroyNotify  destroy_notify);
 
-
-
 #endif /* __PANGOFT2_PRIVATE_H__ */
