@@ -34,6 +34,11 @@
 #include "pangoft2.h"
 #include "pangoft2-private.h"
 
+/* for compatibility with older freetype versions */
+#ifndef FT_LOAD_TARGET_MONO
+#define FT_LOAD_TARGET_MONO  FT_LOAD_MONOCHROME
+#endif
+
 #define PANGO_TYPE_FT2_FONT              (pango_ft2_font_get_type ())
 #define PANGO_FT2_FONT(object)           (G_TYPE_CHECK_INSTANCE_CAST ((object), PANGO_TYPE_FT2_FONT, PangoFT2Font))
 #define PANGO_FT2_FONT_CLASS(klass)      (G_TYPE_CHECK_CLASS_CAST ((klass), PANGO_TYPE_FT2_FONT, PangoFT2FontClass))
@@ -66,29 +71,30 @@ static PangoFontClass *parent_class;	/* Parent class structure for PangoFT2Font
 
 static void pango_ft2_font_class_init (PangoFT2FontClass *class);
 static void pango_ft2_font_init       (PangoFT2Font      *ft2font);
-static void pango_ft2_font_dispose    (GObject         *object);
 static void pango_ft2_font_finalize   (GObject         *object);
 
-static PangoFontDescription *pango_ft2_font_describe          (PangoFont            *font);
+static PangoFontDescription *pango_ft2_font_describe          (PangoFont      *font);
 
-static PangoEngineShape *    pango_ft2_font_find_shaper       (PangoFont            *font,
-							       PangoLanguage        *language,
-							       guint32               ch);
+static PangoEngineShape *    pango_ft2_font_find_shaper       (PangoFont      *font,
+							       PangoLanguage  *language,
+							       guint32         ch);
 
-static void                  pango_ft2_font_get_glyph_extents (PangoFont            *font,
-							       PangoGlyph            glyph,
-							       PangoRectangle       *ink_rect,
-							       PangoRectangle       *logical_rect);
+static void                  pango_ft2_font_get_glyph_extents (PangoFont      *font,
+							       PangoGlyph      glyph,
+							       PangoRectangle *ink_rect,
+							       PangoRectangle *logical_rect);
 
-static PangoFontMetrics *    pango_ft2_font_get_metrics       (PangoFont            *font,
-							       PangoLanguage        *language);
+static PangoFontMetrics *    pango_ft2_font_get_metrics       (PangoFont      *font,
+							       PangoLanguage  *language);
   
-static void                  pango_ft2_get_item_properties    (PangoItem      	    *item,
-							       PangoUnderline 	    *uline,
-							       PangoAttrColor 	    *fg_color,
-							       gboolean       	    *fg_set,
-							       PangoAttrColor 	    *bg_color,
-							       gboolean       	    *bg_set);
+static void                  pango_ft2_get_item_properties    (PangoItem      *item,
+							       PangoUnderline *uline,
+							       gboolean       *strikethrough,
+							       gint           *rise,
+							       gboolean       *shape_set,
+							       PangoRectangle *ink_rect,
+							       PangoRectangle *logical_rect);
+
 
 static GType pango_ft2_font_get_type (void);
 
@@ -197,33 +203,42 @@ load_fallback_face (PangoFT2Font *ft2font,
  * Return value: a pointer to a #FT_Face structure, with the size set correctly
  **/
 FT_Face
-pango_ft2_font_get_face (PangoFont      *font)
+pango_ft2_font_get_face (PangoFont *font)
 {
   PangoFT2Font *ft2font = (PangoFT2Font *)font;
   FT_Error error;
   FcPattern *pattern;
   FcChar8 *filename;
-  FcBool hinting, autohint;
+  FcBool antialias, hinting, autohint;
   int id;
 
   pattern = ft2font->font_pattern;
 
   if (!ft2font->face)
     {
-      /* If we add support for not antialising, then we should
-       * need to conditionalize NO_BITMAP on that.
-       */
-      ft2font->load_flags = FT_LOAD_NO_BITMAP;
+      ft2font->load_flags = 0;
+
+      /* disable antialiasing if requested */
+      if (FcPatternGetBool (pattern,
+                            FC_ANTIALIAS, 0, &antialias) != FcResultMatch)
+	antialias = FcTrue;
+
+      if (antialias)
+        ft2font->load_flags |= FT_LOAD_NO_BITMAP;
+      else
+	ft2font->load_flags |= FT_LOAD_TARGET_MONO;
 
       /* disable hinting if requested */
-      if (FcPatternGetBool (pattern, FC_HINTING, 0, &hinting) != FcResultMatch)
+      if (FcPatternGetBool (pattern,
+                            FC_HINTING, 0, &hinting) != FcResultMatch)
 	hinting = FcTrue;
 
       if (!hinting)
         ft2font->load_flags |= FT_LOAD_NO_HINTING;
 
       /* force autohinting if requested */
-      if (FcPatternGetBool (pattern, FC_AUTOHINT, 0, &autohint) != FcResultMatch)
+      if (FcPatternGetBool (pattern,
+                            FC_AUTOHINT, 0, &autohint) != FcResultMatch)
 	autohint = FcFalse;
 
       if (autohint)
@@ -247,7 +262,8 @@ pango_ft2_font_get_face (PangoFont      *font)
       
       if (!set_unicode_charmap (ft2font->face))
 	{
-	  g_warning ("Unable to load unicode charmap from font file %s", filename);
+	  g_warning ("Unable to load unicode charmap from font file %s",
+                     filename);
 	  
 	  FT_Done_Face (ft2font->face);
 	  ft2font->face = NULL;
@@ -315,7 +331,6 @@ pango_ft2_font_class_init (PangoFT2FontClass *class)
   parent_class = g_type_class_peek_parent (class);
   
   object_class->finalize = pango_ft2_font_finalize;
-  object_class->dispose = pango_ft2_font_dispose;
   
   font_class->describe = pango_ft2_font_describe;
   font_class->get_coverage = pango_ft2_font_get_coverage;
@@ -344,11 +359,13 @@ pango_ft2_font_render_glyph (PangoFont *font,
   
   if (face)
     {
-      PangoFT2Font *ft2font = (PangoFT2Font *)font;
-	    
+      PangoFT2Font *ft2font = (PangoFT2Font *) font;
+
       /* Draw glyph */
       FT_Load_Glyph (face, glyph_index, ft2font->load_flags);
-      FT_Render_Glyph (face->glyph, ft_render_mode_normal);
+      FT_Render_Glyph (face->glyph,
+		       (ft2font->load_flags & FT_LOAD_TARGET_MONO ?
+			ft_render_mode_mono : ft_render_mode_normal));
 
       rendered->bitmap = face->glyph->bitmap;
       rendered->bitmap.buffer = g_memdup (face->glyph->bitmap.buffer,
@@ -401,7 +418,8 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 	  PangoFT2RenderedGlyph *rendered_glyph;
 	  glyph_index = gi->glyph;
 
-	  rendered_glyph = pango_ft2_font_get_cache_glyph_data (font, glyph_index);
+	  rendered_glyph = pango_ft2_font_get_cache_glyph_data (font,
+								glyph_index);
 	  add_glyph_to_cache = FALSE;
 	  if (rendered_glyph == NULL)
 	    {
@@ -413,10 +431,12 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 	  iyoff = y + PANGO_PIXELS (gi->geometry.y_offset);
 	  
 	  x_start = MAX (0, - (ixoff + rendered_glyph->bitmap_left));
-	  x_limit = MIN (rendered_glyph->bitmap.width, bitmap->width - (ixoff + rendered_glyph->bitmap_left));
+	  x_limit = MIN (rendered_glyph->bitmap.width,
+			 bitmap->width - (ixoff + rendered_glyph->bitmap_left));
 
 	  y_start = MAX (0,  - (iyoff - rendered_glyph->bitmap_top));
-	  y_limit = MIN (rendered_glyph->bitmap.rows, bitmap->rows - (iyoff - rendered_glyph->bitmap_top));
+	  y_limit = MIN (rendered_glyph->bitmap.rows,
+			 bitmap->rows - (iyoff - rendered_glyph->bitmap_top));
 
 	  PING (("glyph %d:%d: bitmap: %dx%d, left:%d top:%d",
 		 i, glyph_index,
@@ -425,61 +445,79 @@ pango_ft2_render (FT_Bitmap        *bitmap,
 	  PING (("xstart:%d xlim:%d ystart:%d ylim:%d",
 		 x_start, x_limit, y_start, y_limit));
 
-	  if (rendered_glyph->bitmap.pixel_mode == ft_pixel_mode_grays)
-		for (iy = y_start; iy < y_limit; iy++)
-		  {
-		    dest = bitmap->buffer +
-		      (iyoff - rendered_glyph->bitmap_top + iy) * bitmap->pitch +
-		      ixoff + rendered_glyph->bitmap_left + x_start;
-		    
-		    src = rendered_glyph->bitmap.buffer + 
-                      iy * rendered_glyph->bitmap.pitch + x_start;
+	  src = rendered_glyph->bitmap.buffer +
+	    y_start * rendered_glyph->bitmap.pitch;
 
-		    for (ix = x_start; ix < x_limit; ix++)
-		      {
-                        switch (*src)
-                          {
-                          case 0:
-                            break;
-                          case 0xff:
-                            *dest = 0xff;
-                          default:
-                            *dest = MIN ((gushort) *dest + (gushort) *src, 0xff);
-                            break;
-                          }
-                        dest++;
-			src++;
-		      }
-		  }
-	      else if (rendered_glyph->bitmap.pixel_mode == ft_pixel_mode_mono)
-		for (iy = y_start; iy < y_limit; iy++)
-		  {
-		    dest = bitmap->buffer +
-		      (iyoff - rendered_glyph->bitmap_top + iy) * bitmap->pitch +
-		      ixoff + rendered_glyph->bitmap_left + 
-                      x_start;
-                    
-                    src = rendered_glyph->bitmap.buffer + 
-                      iy*rendered_glyph->bitmap.pitch;
+	  dest = bitmap->buffer +
+	    (y_start + iyoff - rendered_glyph->bitmap_top) * bitmap->pitch +
+	    x_start + ixoff + rendered_glyph->bitmap_left;
 
-		    for (ix = x_start; ix < x_limit; ix++)
-		      {
-			if ((*src) & (1 << (7 - (ix % 8))))
-			  *dest |= (1 << (7 - (ix % 8)));
-			if ((ix % 8) == 7)
-			  src++;
-			dest++;
-		      }
-		  }
-	      else
-		g_warning ("pango_ft2_render: Unrecognized glyph bitmap pixel mode %d\n",
-			   rendered_glyph->bitmap.pixel_mode);
+	  switch (rendered_glyph->bitmap.pixel_mode)
+	    {
+	    case ft_pixel_mode_grays:
+	      src += x_start;
+	      for (iy = y_start; iy < y_limit; iy++)
+		{
+		  guchar *s = src;
+		  guchar *d = dest;
+
+		  for (ix = x_start; ix < x_limit; ix++)
+		    {
+		      switch (*s)
+			{
+			case 0:
+			  break;
+			case 0xff:
+			  *d = 0xff;
+			default:
+			  *d = MIN ((gushort) *d + (gushort) *s, 0xff);
+			  break;
+			}
+
+		      s++;
+		      d++;
+		    }
+
+		  dest += bitmap->pitch;
+		  src  += rendered_glyph->bitmap.pitch;
+		}
+	      break;
+
+	    case ft_pixel_mode_mono:
+	      src += x_start / 8;
+	      for (iy = y_start; iy < y_limit; iy++)
+		{
+		  guchar *s = src;
+		  guchar *d = dest;
+
+		  for (ix = x_start; ix < x_limit; ix++)
+		    {
+		      if ((*s) & (1 << (7 - (ix % 8))))
+			*d |= 0xff;
+		      
+		      if ((ix % 8) == 7)
+			s++;
+		      d++;
+		    }
+
+		  dest += bitmap->pitch;
+		  src  += rendered_glyph->bitmap.pitch;
+		}
+	      break;
+	      
+	    default:
+	      g_warning ("pango_ft2_render: "
+			 "Unrecognized glyph bitmap pixel mode %d\n",
+			 rendered_glyph->bitmap.pixel_mode);
+	      break;
+	    }
 
 	  if (add_glyph_to_cache)
 	    {
 	      pango_ft2_font_set_glyph_cache_destroy (font,
 						      (GDestroyNotify) pango_ft2_free_rendered_glyph);
-	      pango_ft2_font_set_cache_glyph_data (font, glyph_index, rendered_glyph);
+	      pango_ft2_font_set_cache_glyph_data (font,
+						   glyph_index, rendered_glyph);
 	    }
 	}
 
@@ -668,22 +706,6 @@ pango_ft2_font_get_metrics (PangoFont     *font,
   return pango_font_metrics_ref (info->metrics);
 }
 
-static void
-pango_ft2_font_dispose (GObject *object)
-{
-  PangoFT2Font *ft2font = PANGO_FT2_FONT (object);
-
-  /* If the font is not already in the freed-fonts cache, add it,
-   * if it is already there, do nothing and the font will be
-   * freed.
-   */
-  if (!ft2font->in_cache && ft2font->fontmap)
-    _pango_ft2_font_map_cache_add (ft2font->fontmap, ft2font);
-
-  G_OBJECT_CLASS (parent_class)->dispose (object);
-}
-
-
 static gboolean
 pango_ft2_free_glyph_info_callback (gpointer key, gpointer value, gpointer data)
 {
@@ -848,62 +870,75 @@ pango_ft2_render_layout_line (FT_Bitmap       *bitmap,
 			      int              y)
 {
   GSList *tmp_list = line->runs;
-  PangoRectangle overall_rect;
   PangoRectangle logical_rect;
   PangoRectangle ink_rect;
   int x_off = 0;
 
-  pango_layout_line_get_extents (line,NULL, &overall_rect);
-  
   while (tmp_list)
     {
       PangoUnderline uline = PANGO_UNDERLINE_NONE;
+      gboolean strike, shape_set;
+      gint rise, risen_y;
       PangoLayoutRun *run = tmp_list->data;
-      PangoAttrColor fg_color, bg_color;
-      gboolean fg_set, bg_set;
-      
+
       tmp_list = tmp_list->next;
 
-      pango_ft2_get_item_properties (run->item, &uline, &fg_color, &fg_set, &bg_color, &bg_set);
+      pango_ft2_get_item_properties (run->item,
+				     &uline, &strike, &rise,
+				     &shape_set, &ink_rect, &logical_rect);
 
-      if (uline == PANGO_UNDERLINE_NONE)
-	pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
-				    NULL, &logical_rect);
-      else
-	pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
-				    &ink_rect, &logical_rect);
+      risen_y = y - PANGO_PIXELS (rise);
 
-      pango_ft2_render (bitmap, run->item->analysis.font, run->glyphs,
-			x + PANGO_PIXELS (x_off), y);
+      if (!shape_set)
+	{
+	  if (uline == PANGO_UNDERLINE_NONE)
+	    pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
+					NULL, &logical_rect);
+	  else
+	    pango_glyph_string_extents (run->glyphs, run->item->analysis.font,
+					&ink_rect, &logical_rect);
+
+	  pango_ft2_render (bitmap, run->item->analysis.font, run->glyphs,
+			    x + PANGO_PIXELS (x_off), risen_y);
+	}
 
       switch (uline)
 	{
 	case PANGO_UNDERLINE_NONE:
 	  break;
 	case PANGO_UNDERLINE_DOUBLE:
-	  pango_ft2_draw_hline (bitmap, y + 4,
+	  pango_ft2_draw_hline (bitmap,
+				risen_y + 4,
 				x + PANGO_PIXELS (x_off + ink_rect.x),
 				x + PANGO_PIXELS (x_off + ink_rect.x + ink_rect.width));
 	  /* Fall through */
 	case PANGO_UNDERLINE_SINGLE:
-	  pango_ft2_draw_hline (bitmap, y + 2,
+	  pango_ft2_draw_hline (bitmap,
+				risen_y + 2,
 				x + PANGO_PIXELS (x_off + ink_rect.x),
 				x + PANGO_PIXELS (x_off + ink_rect.x + ink_rect.width));
 	  break;
 	case PANGO_UNDERLINE_LOW:
-	  pango_ft2_draw_hline (bitmap, y + PANGO_PIXELS (ink_rect.y + ink_rect.height),
+	  pango_ft2_draw_hline (bitmap,
+				risen_y + PANGO_PIXELS (ink_rect.y + ink_rect.height),
 				x + PANGO_PIXELS (x_off + ink_rect.x),
 				x + PANGO_PIXELS (x_off + ink_rect.x + ink_rect.width));
 	  break;
 	}
 
+      if (strike)
+	pango_ft2_draw_hline (bitmap,
+			      risen_y + PANGO_PIXELS (logical_rect.y + logical_rect.height / 2),
+			      x + PANGO_PIXELS (x_off + logical_rect.x),
+			      x + PANGO_PIXELS (x_off + logical_rect.x + logical_rect.width));
+
       x_off += logical_rect.width;
     }
 }
 
 /**
  * pango_ft2_render_layout:
- * @bitmap:    a FT_Bitmap to render the line onto
+ * @bitmap:    a FT_Bitmap to render the layout onto
  * @layout:    a #PangoLayout
  * @x:         the X position of the left of the layout (in pixels)
  * @y:         the Y position of the top of the layout (in pixels)
@@ -913,80 +948,35 @@ pango_ft2_render_layout_line (FT_Bitmap       *bitmap,
 void 
 pango_ft2_render_layout (FT_Bitmap   *bitmap,
 			 PangoLayout *layout,
-			 int         x, 
-			 int         y)
+			 int          x, 
+			 int          y)
 {
-  PangoRectangle logical_rect;
-  GSList *tmp_list;
-  PangoAlignment align;
-  int indent;
-  int width;
-  int y_offset = 0;
+  PangoLayoutIter *iter;
 
-  gboolean first = TRUE;
-  
   g_return_if_fail (bitmap != NULL);
-  g_return_if_fail (layout != NULL);
+  g_return_if_fail (PANGO_IS_LAYOUT (layout));
 
-  indent = pango_layout_get_indent (layout);
-  width = pango_layout_get_width (layout);
-  align = pango_layout_get_alignment (layout);
+  iter = pango_layout_get_iter (layout);
 
-  PING (("x:%d y:%d indent:%d width:%d", x, y, indent, width));
-
-  if (width == -1 && align != PANGO_ALIGN_LEFT)
+  do
     {
-      pango_layout_get_extents (layout, NULL, &logical_rect);
-      width = logical_rect.width;
-    }
-  
-  tmp_list = pango_layout_get_lines (layout);
-  while (tmp_list)
-    {
-      PangoLayoutLine *line = tmp_list->data;
-      int x_offset;
+      PangoRectangle   logical_rect;
+      PangoLayoutLine *line;
+      int              baseline;
       
-      pango_layout_line_get_extents (line, NULL, &logical_rect);
-
-      if (width != 1 && align == PANGO_ALIGN_RIGHT)
-	x_offset = width - logical_rect.width;
-      else if (width != 1 && align == PANGO_ALIGN_CENTER)
-	x_offset = (width - logical_rect.width) / 2;
-      else
-	x_offset = 0;
-
-      if (first)
-	{
-	  if (indent > 0)
-	    {
-	      if (align == PANGO_ALIGN_LEFT)
-		x_offset += indent;
-	      else
-		x_offset -= indent;
-	    }
-
-	  first = FALSE;
-	}
-      else
-	{
-	  if (indent < 0)
-	    {
-	      if (align == PANGO_ALIGN_LEFT)
-		x_offset -= indent;
-	      else
-		x_offset += indent;
-	    }
-	}
-
-      PING (("x_offset:%d y_offset:%d logical_rect.y:%d logical_rect.height:%d", x_offset, y_offset, logical_rect.y, logical_rect.height));
-
-      pango_ft2_render_layout_line (bitmap, line,
-				    x + PANGO_PIXELS (x_offset),
-				    y + PANGO_PIXELS (y_offset - logical_rect.y));
-
-      y_offset += logical_rect.height;
-      tmp_list = tmp_list->next;
+      line = pango_layout_iter_get_line (iter);
+      
+      pango_layout_iter_get_line_extents (iter, NULL, &logical_rect);
+      baseline = pango_layout_iter_get_baseline (iter);
+      
+      pango_ft2_render_layout_line (bitmap,
+                                    line,
+				    x + PANGO_PIXELS (logical_rect.x),
+				    y + PANGO_PIXELS (baseline));
     }
+  while (pango_layout_iter_next_line (iter));
+
+  pango_layout_iter_free (iter);
 }
 
 /* This utility function is duplicated here and in pango-layout.c; should it be
@@ -995,19 +985,23 @@ pango_ft2_render_layout (FT_Bitmap   *bitmap,
 static void
 pango_ft2_get_item_properties (PangoItem      *item,
 			       PangoUnderline *uline,
-			       PangoAttrColor *fg_color,
-			       gboolean       *fg_set,
-			       PangoAttrColor *bg_color,
-			       gboolean       *bg_set)
+			       gboolean       *strikethrough,
+                               gint           *rise,
+			       gboolean       *shape_set,
+			       PangoRectangle *ink_rect,
+			       PangoRectangle *logical_rect)
 {
   GSList *tmp_list = item->analysis.extra_attrs;
 
-  if (fg_set)
-    *fg_set = FALSE;
-  
-  if (bg_set)
-    *bg_set = FALSE;
-  
+  if (strikethrough)
+    *strikethrough = FALSE;
+
+  if (rise)
+    *rise = 0;
+
+  if (shape_set)
+    *shape_set = FALSE;
+
   while (tmp_list)
     {
       PangoAttribute *attr = tmp_list->data;
@@ -1019,25 +1013,29 @@ pango_ft2_get_item_properties (PangoItem      *item,
 	    *uline = ((PangoAttrInt *)attr)->value;
 	  break;
 	  
-	case PANGO_ATTR_FOREGROUND:
-	  if (fg_color)
-	    *fg_color = *((PangoAttrColor *)attr);
-	  if (fg_set)
-	    *fg_set = TRUE;
-	  
+	case PANGO_ATTR_STRIKETHROUGH:
+	  if (strikethrough)
+	    *strikethrough = ((PangoAttrInt *)attr)->value;
 	  break;
-	  
-	case PANGO_ATTR_BACKGROUND:
-	  if (bg_color)
-	    *bg_color = *((PangoAttrColor *)attr);
-	  if (bg_set)
-	    *bg_set = TRUE;
-	  
+
+	case PANGO_ATTR_SHAPE:
+	  if (shape_set)
+	    *shape_set = TRUE;
+	  if (logical_rect)
+	    *logical_rect = ((PangoAttrShape *)attr)->logical_rect;
+	  if (ink_rect)
+	    *ink_rect = ((PangoAttrShape *)attr)->ink_rect;
 	  break;
-	  
+
+        case PANGO_ATTR_RISE:
+          if (rise)
+            *rise = ((PangoAttrInt *)attr)->value;
+          break;
+
 	default:
 	  break;
 	}
+
       tmp_list = tmp_list->next;
     }
 }
